/*
 * Copyright (c) 2002 - trem0r
 * This file is part of the PornView - photo collection manager.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <sys/time.h>
#include <sys/types.h>

#include "intl.h"
#include "dialogs.h"
#include "pixbuf_utils.h"
#include "file_utils.h"

#include "pixmaps/alert.xpm"
#include "pixmaps/question.xpm"

/* callback functions for confirm dialog */
static void cb_dialog_confirm_yes (GtkWidget * button, ConfirmType * type);
static void cb_dialog_confirm_no (GtkWidget * button, ConfirmType * type);

/* callback functions for message dialog */
static void cb_dialog_message_quit (GtkWidget * button, gpointer data);

ConfirmType
dialog_confirm (const gchar * title, const gchar * message,
		GtkWidget * parent)
{
    ConfirmType retval = CONFIRM_NO;
    GtkWidget *dialog;
    GtkWidget *vbox, *hbox, *button, *label;
    GtkWidget *icon;

    dialog = gtk_dialog_new ();
    gtk_window_set_title (GTK_WINDOW (dialog), title);

#ifndef USE_GTK2
    GTK_WINDOW (dialog)->type = GTK_WINDOW_DIALOG;
#endif

    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
    gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, FALSE, FALSE);
    gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);

    if (parent)
	gtk_window_set_transient_for (GTK_WINDOW (dialog),
				      GTK_WINDOW (parent));

    gtk_widget_set_usize (GTK_WIDGET (dialog), 350, -1);

    gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
			GTK_SIGNAL_FUNC (cb_dialog_confirm_no), NULL);

    /*
     * message area 
     */
    vbox = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), vbox, TRUE, TRUE,
			0);
    gtk_container_set_border_width (GTK_CONTAINER (vbox), 15);
    gtk_widget_show (vbox);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
    gtk_widget_show (hbox);

    /*
     * icon 
     */
    icon = pixbuf_create_pixmap_from_xpm_data (question_xpm);
    gtk_box_pack_start (GTK_BOX (hbox), icon, TRUE, TRUE, 0);
    gtk_widget_show (icon);

    /*
     * message 
     */
    label = gtk_label_new (message);
    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
    gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
    gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
    gtk_widget_set_usize (GTK_WIDGET (label), 220, -1);
    gtk_widget_show (label);

#ifdef USE_GTK2
    gtk_button_box_set_layout (GTK_BUTTON_BOX
			       (GTK_DIALOG (dialog)->action_area),
			       GTK_BUTTONBOX_END);
#endif

    /*
     * buttons
     */
    button = gtk_button_new_with_label (_("Yes"));
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
			button, FALSE, FALSE, 0);
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (cb_dialog_confirm_yes), &retval);
    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
    gtk_widget_show (button);

#ifndef USE_GTK2
    gtk_widget_set_usize (GTK_WIDGET (button), 150, 35);
#endif

    button = gtk_button_new_with_label (_("No"));
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
			button, FALSE, FALSE, 0);
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (cb_dialog_confirm_no), &retval);
    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);

#ifndef USE_GTK2
    gtk_widget_set_usize (GTK_WIDGET (button), 150, 35);
#endif

    gtk_widget_show (button);

    gtk_widget_grab_default (button);

    gtk_widget_show (dialog);

    gtk_grab_add (dialog);
    gtk_main ();
    gtk_grab_remove (dialog);

    gtk_widget_destroy (dialog);

    return retval;
}

void
dialog_message (const gchar * title, const gchar * message,
		GtkWidget * parent)
{
    GtkWidget *dialog;
    GtkWidget *dialog_vbox;
    GtkWidget *vbox;
    GtkWidget *hbox;
    GtkWidget *label;
    GtkWidget *dialog_action_area;
    GtkWidget *button;
    GtkWidget *icon;

    dialog = gtk_dialog_new ();
    gtk_object_set_data (GTK_OBJECT (dialog), "dialog", dialog);
    gtk_window_set_title (GTK_WINDOW (dialog), title);

#ifndef USE_GTK2
    GTK_WINDOW (dialog)->type = GTK_WINDOW_DIALOG;
#endif

    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
    gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, FALSE, FALSE);
    gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
    gtk_widget_set_usize (GTK_WIDGET (dialog), 350, -1);

    if (parent)
	gtk_window_set_transient_for (GTK_WINDOW (dialog),
				      GTK_WINDOW (parent));

    gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
			GTK_SIGNAL_FUNC (gtk_widget_destroy), dialog);

    dialog_vbox = GTK_DIALOG (dialog)->vbox;
    gtk_object_set_data (GTK_OBJECT (dialog), "dialog_vbox", dialog_vbox);
    gtk_widget_show (dialog_vbox);

    /*
     * message area 
     */
    vbox = gtk_vbox_new (FALSE, 0);
    gtk_widget_ref (vbox);
    gtk_object_set_data_full (GTK_OBJECT (dialog), "vbox", vbox,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (vbox);
    gtk_box_pack_start (GTK_BOX (dialog_vbox), vbox, TRUE, TRUE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (vbox), 15);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
    gtk_widget_show (hbox);

    /*
     * icon 
     */
    icon = pixbuf_create_pixmap_from_xpm_data (alert_xpm);
    gtk_box_pack_start (GTK_BOX (hbox), icon, TRUE, TRUE, 0);
    gtk_widget_show (icon);

    /*
     * message 
     */
    label = gtk_label_new (message);
    gtk_widget_ref (label);
    gtk_object_set_data_full (GTK_OBJECT (dialog), "label", label,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (label);
    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
    gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
    gtk_widget_set_usize (GTK_WIDGET (label), 220, -1);
    gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);

    dialog_action_area = GTK_DIALOG (dialog)->action_area;
    gtk_object_set_data (GTK_OBJECT (dialog), "dialog_action_area",
			 dialog_action_area);
    gtk_widget_show (dialog_action_area);
    gtk_container_set_border_width (GTK_CONTAINER (dialog_action_area), 5);

#ifdef USE_GTK2
    gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area),
			       GTK_BUTTONBOX_SPREAD);
#endif

    button = gtk_button_new_with_label (_("Close"));
    gtk_widget_ref (button);
    gtk_object_set_data_full (GTK_OBJECT (dialog), "button", button,
			      (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (button);

#ifndef USE_GTK2
    gtk_widget_set_usize (GTK_WIDGET (button), 150, 35);
#else
    gtk_widget_set_usize (GTK_WIDGET (button), 150, -2);
#endif

    gtk_box_pack_start (GTK_BOX (dialog_action_area), button, FALSE, FALSE,
			0);

    gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (cb_dialog_message_quit), dialog);
    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);

    gtk_widget_grab_default (button);

    gtk_widget_show (dialog);
}

GtkWidget *
dialog_choose_dir (gchar * title)
{
    GtkWidget *dialog;

    dialog = gtk_file_selection_new (title);

#ifndef USE_GTK2
    GTK_WINDOW (dialog)->type = GTK_WINDOW_DIALOG;
#endif

    gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, FALSE, FALSE);
    gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);

    gtk_widget_hide (GTK_FILE_SELECTION (dialog)->file_list->parent);
    gtk_widget_hide (GTK_FILE_SELECTION (dialog)->fileop_del_file);
    gtk_widget_hide (GTK_FILE_SELECTION (dialog)->fileop_ren_file);
    gtk_widget_hide (GTK_FILE_SELECTION (dialog)->fileop_c_dir);
    gtk_widget_hide (GTK_FILE_SELECTION (dialog)->selection_entry);
    gtk_widget_hide (GTK_FILE_SELECTION (dialog)->selection_text);

    return dialog;
}

static void
cb_dialog_confirm_yes (GtkWidget * button, ConfirmType * type)
{
    *type = CONFIRM_YES;
    gtk_main_quit ();
}

static void
cb_dialog_confirm_no (GtkWidget * button, ConfirmType * type)
{
    *type = CONFIRM_NO;
    gtk_main_quit ();
}

static void
cb_dialog_message_quit (GtkWidget * button, gpointer data)
{
    gtk_widget_destroy (data);
}
